/* ---------------------------------------------------- */
/*    основной файл обработчика энкодера encod.cpp      */
//******************************************************************************
//  Секция include: здесь подключается заголовочный файл к модулю
//******************************************************************************

#include "encod.h" // заголовочный *.h файл текущего проекта

//------------------------------------------------------------------------

// указатель на текущую исполняемую функцию
// анализатора состояния энкодера
volatile Fn0Ptr encPtr;

// байт контрольных данных (рвх комбинации)
//byte encKey;
// вспомогательная "времянка"
//volatile byte encTmp;
// внутренний счетчик интервала между опросами
// линий энкодера
volatile byte enc_wait_cnt;

// флаг вспомогательного генератора таймер-меток
volatile byte QtTm;

// флаг запроса очередной обработки данных энкодера
// (содержимого счетчика enc_cnt)
volatile byte QtEnc;

// основной счетчик энкодера
volatile int enc_cnt;

// флаг энкодера
volatile byte enc_F;

//------------------------------------------------------------------------

// инициалзация выводов МК и генератора сетки опроса сигнальных линий
void enc_init()
{
 pinMode(P_enc, INPUT_PULLUP); pinMode(D_enc, INPUT_PULLUP);
 digitalWrite(P_enc, HIGH); digitalWrite(D_enc, HIGH);
 enc_wait_cnt = enc_wait; enc_cnt = 0; enc_F = 0;
 encPtr = enc_start; QtEnc =0;
 cli(); //noInterrupts();
 OCR0A = 127; TIMSK0 |= (1<<OCIE0A); QtTm = 0;
 sei(); //interrupts();
}

//------------------------------------------------------------------------

/* 
 *  генератор сетки опроса обработчика энкодера (1mS)
 *  обьявление функции в файле avr/interrupt.h
*/

ISR(TIMER0_COMPA_vect)
{
 // генератор интервала между опросами PortEnc
 //в enc_wait микросекундах
 enc_wait_cnt ++;
 if (enc_wait_cnt == enc_wait)
  {
   enc_wait_cnt = 0; QtTm = 1;
   // чтение и нормирование сигналов энкодера (lin_dat)
   byte enc_tmp;
   enc_tmp = ((PortEnc >> 6) & B00000011) | B00000100;
   // выполнение следующей ступени анализатора
   encPtr(enc_tmp);
  }
}

//------------------------------------------------------------------------
// функция определения направления
void enc_start(byte tmpp)
{
 switch(tmpp){
  case B00000100:
    // невозможно при исправном энкодере
   break;
  case B00000110:
    // P вращение начало
    encPtr = enc_P1;
   break;
  case B00000101:
    // D вращение начало
    encPtr = enc_D1;
   break;
  case B00000111:
    // простой в исходной позиции в ожидании вращения
    encPtr = enc_start; enc_F = 0;
   break;
  default: 
      // if nothing else matches, do the default
      // default is optional
   break;
 }
}

// функция первой ступени P1
void enc_P1(byte tmpp)
{
 switch(tmpp){
  case B00000100:
    // если enc_F =0
    // инкремент счетчика enc_cnt и enc_F =1
    // и переход к enc_P2
    if (!enc_F)
     { enc_cnt ++; enc_F = 1; }
    encPtr = enc_P2;
   break;
  case B00000110:
    // простой в текущем положении
    encPtr = enc_P1;
   break;
  case B00000101:
    // невозможно при исправном энкодере
   break;
  case B00000111:
    // возврат в предыдущее положение
    encPtr = enc_start;
   break;
  default: 
      // if nothing else matches, do the default
      // default is optional
   break;
 }
}

// функция первой ступени D1
void enc_D1(byte tmpp)
{
 switch(tmpp){
  case B00000100:
    // если enc_F =0
    // декремент счетчика enc_cnt и enc_F =1
    // и переход к enc_D2
    if (!enc_F)
     { enc_cnt --; enc_F = 1; }
    encPtr = enc_D2;
   break;
  case B00000110:
    // невозможно при исправном энкодере
   break;
  case B00000101:
    // простой в текущем положении
    encPtr = enc_D1;
   break;
  case B00000111:
    // возврат в предыдущее положение
    encPtr = enc_start;
   break;
  default: 
      // if nothing else matches, do the default
      // default is optional
   break;
 }
}

// функция второй ступени P2
void enc_P2(byte tmpp)
{
 switch(tmpp){
  case B00000100:
    // простой в текущем положении
    encPtr = enc_P2;
   break;
  case B00000110:
    // возврат в предыдущее положение
    enc_cnt --; enc_F = 0;
    encPtr = enc_P1;
   break;
  case B00000101:
    // переход к enc_P3
    encPtr = enc_P3;
   break;
  case B00000111:
    // невозможно при исправном энкодере
   break;
  default: 
      // if nothing else matches, do the default
      // default is optional
   break;
 }
}

// функция второй ступени D2
void enc_D2(byte tmpp)
{
 switch(tmpp){
  case B00000100:
    // простой в текущем положении
    encPtr = enc_D2;
   break;
  case B00000110:
    // переход к enc_D3
    encPtr = enc_D3;
   break;
  case B00000101:
    // возврат в предыдущее положение
    enc_cnt ++; enc_F = 0;
    encPtr = enc_D1;
   break;
  case B00000111:
    // невозможно при исправном энкодере
   break;
  default: 
      // if nothing else matches, do the default
      // default is optional
   break;
 }
}

// функция третьей ступени P3
void enc_P3(byte tmpp)
{
 switch(tmpp){
  case B00000100:
    // возврат в предыдущее положение
    encPtr = enc_P2;
   break;
  case B00000110:
    // невозможно при исправном энкодере
   break;
  case B00000101:
    // простой в текущем положении
    encPtr = enc_P3;
   break;
  case B00000111:
    // переход к завершению цикла с QtEnc = 1
    enc_F = 0; QtEnc = 1;
    encPtr = enc_start;
   break;
  default: 
      // if nothing else matches, do the default
      // default is optional
   break;
 }
}

// функция третьей ступени D3
void enc_D3(byte tmpp)
{
 switch(tmpp){
  case B00000100:
    // возврат в предыдущее положение
    encPtr = enc_D2;
   break;
  case B00000110:
    // простой в текущем положении
    encPtr = enc_D3;
   break;
  case B00000101:
    // невозможно при исправном энкодере
   break;
  case B00000111:
    // переход к завершению цикла с QtEnc = 1
    enc_F = 0; QtEnc = 1;
    encPtr = enc_start;
   break;
  default: 
      // if nothing else matches, do the default
      // default is optional
   break;
 }
}

//------------------------------------------------------------------------
//-------------конец файла/end of file---------------------
