/* 
 *  вариант дисплея на основе MAX7219
 *            файл max72m.h
 */
// защита от переоперделения с заключительным
// #endif в конце файла
#ifndef MAX72M_H
#define MAX72M_H

//------------------
#if ARDUINO >= 100
 #include "Arduino.h"
#else
 #include "WProgram.h"
#endif
//------------------
// добавка для удобства работы с многофайловиками
// #include "dhsct.h"
//------------------

// собственно описание класса

class m72
{
  public:
  
  // объявление конструктора
  
   m72 (byte, byte, byte);
/*
 dn - значение линии данных интерфейса дисплея
              активный уровень равен 1
 clk - значение линии строба интерфейса дисплея
             активный уровень равен 1, защелкивание данных по переднему
             (наростающему) фронту импульса строба
 cs - значение линии выборки кристалла интерфейса дисплея
            активный уровень равен 0
ограничений на максимальную длительность импульсов clc не оговорено
минимальная длительность clk = 50nS
задержка начала/завершения пакета относительно cs не менее 50nS
предварительное гашение индикации перед сменой данных в ОЗУ ИС
НЕ ТРЕБУЕТСЯ - MAX7219 имеет аппаратную поддержку "беспроблесковой"
смены данных отображения
*/
   
 // объявление открытых членов класса

 // начальная инициализация открытых членов класса

// набор MAX7219 8-позиционник

#define s_A 6 // значение номера сегмента A
#define s_B 5 // значение номера сегмента B
#define s_C 4 // значение номера сегмента C
#define s_D 3 // значение номера сегмента D
#define s_E 2 // значение номера сегмента E
#define s_F 1 // значение номера сегмента F
#define s_G 0 // значение номера сегмента G
#define s_H 7 // значение номера сегмента H

/* "кодовая страница кракозябр 7-сегментных"
 раскладка сегментов по символам определяется стандартной разметкой A-H
 по условию, что активный уровень(сегмент астивен/светится) принят за 1 */

#define fnt_bl 0
#define fnt_0 (1<<s_A | 1<<s_B | 1<<s_C | 1<<s_D | 1<<s_E | 1<<s_F) // цифра 0 или символ "О"
#define fnt_1 (1<<s_B | 1<<s_C) // цифра 1
#define fnt_2 (1<<s_A | 1<<s_B | 1<<s_D | 1<<s_E | 1<<s_G) // цифра 2
#define fnt_3 (1<<s_A | 1<<s_B | 1<<s_C | 1<<s_D | 1<<s_G) // цифра 3 или символ "Э/З"
#define fnt_4 (1<<s_B | 1<<s_C | 1<<s_F | 1<<s_G) // цифра 4 или символ "Ч"
#define fnt_5 (1<<s_A | 1<<s_C | 1<<s_D | 1<<s_F | 1<<s_G) // цифра 5 или символ "S"
#define fnt_6 (1<<s_A | 1<<s_C | 1<<s_D | 1<<s_E | 1<<s_F | 1<<s_G) // цифра 6
#define fnt_7 (1<<s_A | 1<<s_B | 1<<s_C) // цифра 7
#define fnt_8 (1<<s_A | 1<<s_B | 1<<s_C | 1<<s_D | 1<<s_E | 1<<s_F | 1<<s_G) // цифра 8
#define fnt_9 (1<<s_A | 1<<s_B | 1<<s_C | 1<<s_D | 1<<s_F | 1<<s_G) // цифра 9
#define fnt_A (1<<s_A | 1<<s_B | 1<<s_C | 1<<s_E | 1<<s_F | 1<<s_G) // символ "A"
#define fnt_b (1<<s_C | 1<<s_D | 1<<s_E | 1<<s_F | 1<<s_G) // символ "b"
#define fnt_C (1<<s_A | 1<<s_D | 1<<s_E | 1<<s_F) // символ "C" (прямая скобка '[' )
#define fnt_d (1<<s_B | 1<<s_C | 1<<s_D | 1<<s_E | 1<<s_G) // символ "d"
#define fnt_E (1<<s_A | 1<<s_D | 1<<s_E | 1<<s_F | 1<<s_G) // символ "E"
#define fnt_F (1<<s_A | 1<<s_E | 1<<s_F | 1<<s_G) // символ "F"
#define fnt_P (1<<s_A | 1<<s_E | 1<<s_F | 1<<s_G | 1<<s_B) // символ "P"
#define fnt_L (1<<s_E | 1<<s_F | 1<<s_D) // символ "L"
#define fnt_H (1<<s_B | 1<<s_C | 1<<s_E | 1<<s_F | 1<<s_G) // символ "H"
#define fnt_U (1<<s_B | 1<<s_C | 1<<s_E | 1<<s_F | 1<<s_D) // символ "U"
#define fnt_I (1<<s_E | 1<<s_F) // левая 1 или латинская I
#define fnt_S (1<<s_A | 1<<s_C | 1<<s_D | 1<<s_F | 1<<s_G) // аналог цифры 5
#define fnt_J (1<<s_A | 1<<s_B | 1<<s_C | 1<<s_D | 1<<s_E) // символ "J"
#define fnt_G (1<<s_A | 1<<s_C | 1<<s_D | 1<<s_E | 1<<s_F) // символ "G"
#define fnt_r (1<<s_G | 1<<s_E) // символ "r"
#define fnt_n (1<<s_G | 1<<s_E | 1<<s_C) // символ "п"
#define fnt_c (1<<s_G | 1<<s_E | 1<<s_D) // символ "с" или '<'
#define fnt_o (1<<s_G | 1<<s_E | 1<<s_D | 1<<s_C) // нижний кружок "о"
#define fnt_u (1<<s_C | 1<<s_E | 1<<s_D) // символ "u"
#define fnt_h (1<<s_C | 1<<s_E | 1<<s_F | 1<<s_G) // символ "h"
#define fnt_rusg (1<<s_A | 1<<s_E | 1<<s_F) // символ "Г"
#define fnt_rusP (1<<s_A | 1<<s_E | 1<<s_F | 1<<s_B | 1<<s_C) // символ "П"
#define fnt_rus_iE (1<<s_A | 1<<s_B | 1<<s_C | 1<<s_D | 1<<s_G) // символ Э/З/цифра 3 (инверсное Е)
#define fnt_rusY (1<<s_F | 1<<s_G | 1<<s_B | 1<<s_C | 1<<s_D) // символ "У"
#define fnt_qest (1<<s_A | 1<<s_B | 1<<s_E | 1<<s_G) // символ "?"
#define fnt_rC (1<<s_A | 1<<s_B | 1<<s_C | 1<<s_D) // символ ']' (обратная скобка/"вывернутая С")
#define fnt_rc (1<<s_G | 1<<s_C | 1<<s_D) // символ '>'
#define fnt_gradus (1<<s_A | 1<<s_B | 1<<s_F | 1<<s_G) // верхний кружок "символ грвдуса"
#define fnt_minus (1<<s_G) // символ "-" (средняя черта)
#define fnt_aplin (1<<s_A) // символ "верхняя черта"
#define fnt_dnlin (1<<s_D) // символ "_" (нижняя черта)
#define fnt_trlin (1<<s_A | 1<<s_G | 1<<s_D) // символ "три черты"
#define fnt_coma (1<<s_H) // символ "," (децимальная точка)

#define B_fnt_0 0 // значение данных соответствующее символу 0 режима декодера
#define B_fnt_1 1 // значение данных соответствующее символу 1 режима декодера
#define B_fnt_2 2 // значение данных соответствующее символу 2 режима декодера
#define B_fnt_3 3 // значение данных соответствующее символу 3 режима декодера
#define B_fnt_4 4 // значение данных соответствующее символу 4 режима декодера
#define B_fnt_5 5 // значение данных соответствующее символу 5 режима декодера
#define B_fnt_6 6 // значение данных соответствующее символу 6 режима декодера
#define B_fnt_7 7 // значение данных соответствующее символу 7 режима декодера
#define B_fnt_8 8 // значение данных соответствующее символу 8 режима декодера
#define B_fnt_9 9 // значение данных соответствующее символу 9 режима декодера
#define B_fnt_minus 0x0A // значение данных соответствующее символу minus режима декодера
#define B_fnt_E 0x0B // значение данных соответствующее символу E режима декодера
#define B_fnt_H 0x0C // значение данных соответствующее символу H режима декодера
#define B_fnt_L 0x0D // значение данных соответствующее символу L режима декодера
#define B_fnt_P 0x0E // значение данных соответствующее символу P режима декодера
#define B_fnt_bl 0x0F // значение данных соответствующее символу blank режима декодера

#define addp0 1 // значение адреса регистра данных позиции 1 дисплея
#define addp1 2 // значение адреса регистра данных позиции 2 дисплея
#define addp2 3 // значение адреса регистра данных позиции 3 дисплея
#define addp3 4 // значение адреса регистра данных позиции 4 дисплея
#define addp4 5 // значение адреса регистра данных позиции 5 дисплея
#define addp5 6 // значение адреса регистра данных позиции 6 дисплея
#define addp6 7 // значение адреса регистра данных позиции 7 дисплея
#define addp7 8 // значение адреса регистра данных позиции 8 дисплея

#define No_Op 0 // значение адреса регистра No_Op
// регистр No_Op используется исключительно при каскадировании max7219
#define Dec_Mod 9// значение адреса регистра Decode_Mode
#define Intensit 0x0A // значение адреса регистра Intensity
#define Sc_Lim 0x0B // значение адреса регистра Scan_Limit
#define Shut_Dn 0x0C // значение адреса регистра Shut_Down
#define Disp_ts 0x0F // значение адреса регистра Display_test

#define Com_DM_0 0 // Dec_Mod, все позиции только под знакосинтез DM отключен
#define Com_DM_1 1 // Dec_Mod, B_fntN только для позиции 0, поз. 1-7 под знакосинтез
#define Com_DM_2 2 // Dec_Mod, B_fntN для позиций 0-3, поз. 4-7 под знакосинтез
#define Com_DM_3 3 // Dec_Mod, B_fntN для позиций 0-7

#define Com_int_0 0 // Intensit, уровень яркости 1/32 (минимальный)
#define Com_int_1 1 // Intensit, уровень яркости 3/32
#define Com_int_2 2 // Intensit, уровень яркости 5/32
#define Com_int_3 3 // Intensit, уровень яркости 7/32
#define Com_int_4 4 // Intensit, уровень яркости 9/32
#define Com_int_5 5 // Intensit, уровень яркости 11/32
#define Com_int_6 6 // Intensit, уровень яркости 13/32
#define Com_int_7 7 // Intensit, уровень яркости 15/32
#define Com_int_8 8 // Intensit, уровень яркости 17/32
#define Com_int_9 9 // Intensit, уровень яркости 19/32
#define Com_int_A 10 // Intensit, уровень яркости 21/32
#define Com_int_B 11 // Intensit, уровень яркости 23/32
#define Com_int_C 12 // Intensit, уровень яркости 25/32
#define Com_int_D 13 // Intensit, уровень яркости 27/32
#define Com_int_E 14 // Intensit, уровень яркости 29/32
#define Com_int_F 15 // Intensit, уровень яркости 31/32 (максимальный)

#define Com_SL_0 0 // Sc_Lim, индикация только в позиции 0 (max s I 10mA!)
#define Com_SL_1 1 // Sc_Lim, индикация только в позиции 0,1 (max s I 20mA!)
#define Com_SL_2 2 // Sc_Lim, индикация только в позиции 0,1,2 (max s I 30mA!)
#define Com_SL_3 3 // Sc_Lim, индикация только в позиции 0,1,2,3
#define Com_SL_4 4 // Sc_Lim, индикация только в позиции 0,1,2,3,4
#define Com_SL_5 5 // Sc_Lim, индикация только в позиции 0,1,2,3,4,5
#define Com_SL_6 6 // Sc_Lim, индикация только в позиции 0,1,2,3,4,5,6
#define Com_SL_7 7 // Sc_Lim, индикация только в позиции 0,1,2,3,4,5,6,7

#define Com_Shutdn_md 0 // Shut_Dn, значение данных команды Shutdown_mode
#define Com_Normal_md 1 // Shut_Dn, значение данных команды Normal_mode
#define Com_tst_on_md 1 // Disp_ts, значение данных команды display test on_mode
#define Com_tst_of_md 0 // Disp_ts, значение данных команды display test off_mode


/*
 *    v_ram
 * буфер-массив видеопамяти дисплея MAX7219
 * развертка позиций справа налево (7:6:5:4:3:2:1:0)
 */
  byte v_ram[8];

 // объявление функций внешнего доступа
  void init(); // блок инициализации линий управления
  void write(byte, byte); // ( cmda, dat ) запись двойного слова из командного
               // заголовка(адреса назначения или cmda) и собственно данных dat
  void write_s(); // запись строки кодов символов из v_ram в ОЗУ индикатора
  byte znak(byte); // утилитка знакогенератора
  byte overZnak(byte); // "обратный конвертор"
               // сегментного кода из v_ram в байт двоичного кода 0-F
 private:
  
  // объявление закрытых членов класса
  
   byte _ln_dn, _ln_clk, _ln_cs;
   void clks(); // строб сопровождения данных для wr_m72
   
}; // обязательное заключительное ";"

// объявление объекта my72 класса m72 как extern
extern m72 my72;

//------------------------------------------------------------------------
#endif      // Закрывающий #endif к блокировке повторного включения

//-------------конец файла/end of file---------------------
